import pytest
import os, sys
from unittest.mock import patch, MagicMock
from dotenv import load_dotenv
load_dotenv()

# 将项目根目录添加到sys.path中
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from unionllm.exceptions import ProviderError
from unionllm import unionchat
from unionllm.utils import ModelResponse, Message, Choices, Usage, Context

common_messages = [{"content": "你的开发者是谁？", "role": "user"}]

@pytest.mark.parametrize("provider, model, messages, expected_exception, stream_mode", [
    # non stream mode
    ("azure", "azure/gpt-35-turbo", common_messages, None, False),
    ("zhipuai", "glm-4", common_messages, None, False),
    ("tiangong", "SkyChat-MegaVerse", common_messages, None, False),
    ("wenxin", "ERNIE-3.5-8K", common_messages, None, False),
    ("qwen", "qwen-plus", common_messages, None, False),
    ("moonshot", "moonshot-v1-8k", common_messages, None, False),
    ("minimax", "abab5.5-chat", common_messages, None, False),
    ("baichuan", "Baichuan2-Turbo", common_messages, None, False),
    ("xunfei", "generalv3", common_messages, None, False),
    ("lingyi", "yi-large", common_messages, None, False),
    ("stepfun", "step-1-8k", common_messages, None, False),
    ("coze", "coze", common_messages, None, False),
    ("dify", "dify", common_messages, None, False),
    ("fastgpt", "fastgpt", common_messages, None, False),    
    {"ollama", "ollama/llama3", common_messages, None, False}
    ("mistral", "mistral-large-latest", common_messages, None, False),
    ("cohere", "command-r", common_messages, None, False),
    ("groq", "groq/mixtral-8x7b-32768", common_messages, None, False),
    ("bedrock", "ai21.j2-ultra-v1", common_messages, None, False),
    ("", "groq/mixtral-8x7b-32768", common_messages, None, False), #no provider case
    ("nonexistent", "model", common_messages, ProviderError, False),
    # stream mode
    ("azure", "azure/gpt-35-turbo", common_messages, None, True),
    ("zhipuai", "glm-4", common_messages, None, True),
    ("nonexistent", "model", common_messages, ProviderError, True),
    ("tiangong", "SkyChat-MegaVerse", common_messages, None, True),
    ("wenxin", "ERNIE-3.5-8K", common_messages, None, True),
    ("qwen", "qwen-plus", common_messages, None, True),
    ("moonshot", "moonshot-v1-8k", common_messages, None, True),
    ("minimax", "abab5.5-chat", common_messages, None, True),
    ("baichuan", "Baichuan2-Turbo", common_messages, None, True),
    ("coze", "coze", common_messages, None, True),
    ("dify", "dify", common_messages, None, True),
    ("fastgpt", "fastgpt", common_messages, None, True),    
    {"ollama", "ollama/llama3", common_messages, None, False}
    ("mistral", "mistral-large-latest", common_messages, None, True),
    ("cohere", "command-r", common_messages, None, True),
    ("groq", "groq/mixtral-8x7b-32768", common_messages, None, True),
    ("bedrock", "ai21.j2-ultra-v1", common_messages, None, True),
    ("nonexistent", "model", common_messages, ProviderError, True),
    ("lingyi", "yi-large", common_messages, None, True),
    ("stepfun", "step-1-8k", common_messages, None, True),
])

def test_unionchat(provider, model, messages, expected_exception, stream_mode, mocker):
    mock_unionllm = mocker.patch('unionllm.UnionLLM')
    if expected_exception:
        mock_unionllm.return_value.completion.side_effect = ProviderError("Provider not supported")
    else:
        if stream_mode:
            def stream_response():
                yield ModelResponse(id='8635254124951169203', choices=[Choices(finish_reason='stop', index=0, message=Message(content='我是人工智能助手。', role='assistant'))], created=1715570856, model=model, object='chat.completion', system_fingerprint=None, usage=Usage(prompt_tokens=9, completion_tokens=27, total_tokens=36))
            mock_unionllm.return_value.completion.return_value = stream_response()
        else:
            mock_unionllm.return_value.completion.return_value = ModelResponse(id='8635254124951169203', choices=[Choices(finish_reason='stop', index=0, message=Message(content='我是人工智能助手。', role='assistant'))], created=1715570856, model=model, object='chat.completion', system_fingerprint=None, usage=Usage(prompt_tokens=9, completion_tokens=27, total_tokens=36))

    if expected_exception:
        with pytest.raises(expected_exception):
            unionchat(provider=provider, model=model, messages=messages, stream=stream_mode)
    else:
        result = unionchat(provider=provider, model=model, messages=messages, stream=stream_mode)
        if stream_mode:
            for response in result:
                print(response)
                assert isinstance(response, ModelResponse), "Each item in the stream should be an instance of ModelResponse"
        else:
            print(result)
            assert isinstance(result, ModelResponse), "The result should be an instance of ModelResponse"


import json
from abc import ABC, abstractmethod
from openai import OpenAIError as OriginalError
from typing import List, Union, Optional

import uuid, time, openai, random, requests
from openai._models import BaseModel as OpenAIObject

class Message(OpenAIObject):
    def __init__(
        self,
        content="default",
        role="assistant",
        logprobs=None,
        function_call=None,
        **params
    ):
        super(Message, self).__init__(**params)
        self.content = content
        self.role = role
        self._logprobs = logprobs
        if function_call:
            self.function_call = function_call

    def get(self, key, default=None):
        # Custom .get() method to access attributes with a default value if the attribute doesn't exist
        return getattr(self, key, default)

    def __getitem__(self, key):
        # Allow dictionary-style access to attributes
        return getattr(self, key)

    def __setitem__(self, key, value):
        # Allow dictionary-style assignment of attributes
        setattr(self, key, value)


class Delta(OpenAIObject):
    def __init__(self, content=None, role=None, **params):
        super(Delta, self).__init__(**params)
        if content is not None:
            self.content = content
        if role:
            self.role = role

    def __contains__(self, key):
        # Define custom behavior for the 'in' operator
        return hasattr(self, key)

    def get(self, key, default=None):
        # Custom .get() method to access attributes with a default value if the attribute doesn't exist
        return getattr(self, key, default)

    def __getitem__(self, key):
        # Allow dictionary-style access to attributes
        return getattr(self, key)

    def __setitem__(self, key, value):
        # Allow dictionary-style assignment of attributes
        setattr(self, key, value)


class Choices(OpenAIObject):
    def __init__(self, finish_reason=None, index=0, message=None, **params):
        super(Choices, self).__init__(**params)
        if finish_reason:
            self.finish_reason = map_finish_reason(finish_reason)
        else:
            self.finish_reason = "stop"
        self.index = index
        if message is None:
            self.message = Message(content=None)
        else:
            self.message = message

    def __contains__(self, key):
        # Define custom behavior for the 'in' operator
        return hasattr(self, key)

    def get(self, key, default=None):
        # Custom .get() method to access attributes with a default value if the attribute doesn't exist
        return getattr(self, key, default)

    def __getitem__(self, key):
        # Allow dictionary-style access to attributes
        return getattr(self, key)

    def __setitem__(self, key, value):
        # Allow dictionary-style assignment of attributes
        setattr(self, key, value)


class Usage(OpenAIObject):
    def __init__(
        self, prompt_tokens=None, completion_tokens=None, total_tokens=None, **params
    ):
        super(Usage, self).__init__(**params)
        if prompt_tokens:
            self.prompt_tokens = prompt_tokens
        if completion_tokens:
            self.completion_tokens = completion_tokens
        if total_tokens:
            self.total_tokens = total_tokens

    def __contains__(self, key):
        # Define custom behavior for the 'in' operator
        return hasattr(self, key)

    def get(self, key, default=None):
        # Custom .get() method to access attributes with a default value if the attribute doesn't exist
        return getattr(self, key, default)

    def __getitem__(self, key):
        # Allow dictionary-style access to attributes
        return getattr(self, key)

    def __setitem__(self, key, value):
        # Allow dictionary-style assignment of attributes
        setattr(self, key, value)

class Context(OpenAIObject):
    def __init__(self, id=None, content=None, score=None, **params):
        super(Context, self).__init__(**params)
        if id:
            self.id = id
        if content:
            self.content = content
        if score:
            self.score = score

    def __contains__(self, key):
        # Define custom behavior for the 'in' operator
        return hasattr(self, key)

    def get(self, key, default=None):
        # Custom .get() method to access attributes with a default value if the attribute doesn't exist
        return getattr(self, key, default)

    def __getitem__(self, key):
        # Allow dictionary-style access to attributes
        return getattr(self, key)

    def __setitem__(self, key, value):
        # Allow dictionary-style assignment of attributes
        setattr(self, key, value)

class StreamingChoices(OpenAIObject):
    def __init__(
        self, finish_reason=None, index=0, delta: Optional[Delta] = None, **params
    ):
        super(StreamingChoices, self).__init__(**params)
        self.finish_reason = finish_reason
        self.index = index
        if delta:
            self.delta = delta
        else:
            self.delta = Delta()

    def __contains__(self, key):
        # Define custom behavior for the 'in' operator
        return hasattr(self, key)

    def get(self, key, default=None):
        # Custom .get() method to access attributes with a default value if the attribute doesn't exist
        return getattr(self, key, default)

    def __getitem__(self, key):
        # Allow dictionary-style access to attributes
        return getattr(self, key)

    def __setitem__(self, key, value):
        # Allow dictionary-style assignment of attributes
        setattr(self, key, value)


def _generate_id():  # private helper function
    return "chatcmpl-" + str(uuid.uuid4())


def map_finish_reason(
    finish_reason: str,
):  # openai supports 5 stop sequences - 'stop', 'length', 'function_call', 'content_filter', 'null'
    # anthropic mapping
    if finish_reason == "stop_sequence":
        return "stop"
    return finish_reason


class ModelResponse(OpenAIObject):
    id: str
    """A unique identifier for the completion."""

    choices: List[Union[Choices, StreamingChoices]]
    """The list of completion choices the model generated for the input prompt."""

    created: int
    """The Unix timestamp (in seconds) of when the completion was created."""

    model: Optional[str] = None
    """The model used for completion."""

    object: str
    """The object type, which is always "text_completion" """

    system_fingerprint: Optional[str] = None
    """This fingerprint represents the backend configuration that the model runs with.

    Can be used in conjunction with the `seed` request parameter to understand when
    backend changes have been made that might impact determinism.
    """

    usage: Optional[Usage] = None
    """Usage statistics for the completion request."""

    _hidden_params: dict = {}

    def __init__(
        self,
        id=None,
        choices=None,
        created=None,
        model=None,
        object=None,
        system_fingerprint=None,
        usage=None,
        stream=False,
        response_ms=None,
        hidden_params=None,
        **params
    ):
        if stream:
            object = "chat.completion.chunk"
        else:
            object = "chat.completion"
        if id is None:
            id = _generate_id()
        if created is None:
            created = int(time.time())
        if response_ms:
            _response_ms = response_ms
        if not usage:
            usage = Usage()
        if hidden_params:
            self._hidden_params = hidden_params
        super().__init__(
            id=id,
            choices=choices,
            created=created,
            model=model,
            object=object,
            system_fingerprint=system_fingerprint,
            usage=usage,
            **params
        )

    def __contains__(self, key):
        # Define custom behavior for the 'in' operator
        return hasattr(self, key)

    def get(self, key, default=None):
        # Custom .get() method to access attributes with a default value if the attribute doesn't exist
        return getattr(self, key, default)

    def __getitem__(self, key):
        # Allow dictionary-style access to attributes
        return getattr(self, key)

    def __setitem__(self, key, value):
        # Allow dictionary-style assignment of attributes
        setattr(self, key, value)

class EmbeddingResponse(OpenAIObject):
    def __init__(
        self,
        id=None,
        choices=None,
        created=None,
        model=None,
        usage=None,
        stream=False,
        response_ms=None,
        **params
    ):
        self.object = "list"
        if response_ms:
            self._response_ms = response_ms
        else:
            self._response_ms = None
        self.data = []
        self.model = model

    def to_dict_recursive(self):
        d = super().to_dict_recursive()
        return d

def generate_unique_uid():
    # Get the current time in microseconds
    microseconds = int(time.time() * 1000000)
    
    # Get a random number between 0 and 0xFFFF
    rand_num = random.randint(0, 0xFFFF)
    
    # Combine them to get a unique ID
    unique_uid = f"{microseconds:x}{rand_num:04x}"
    
    return unique_uid